There's a problem as old as the web itself: your user needs to be able to bookmark certain search results on the page they want with complicated parameters.  In the past, we threw that stuff in a query string, let the server read the query string and restore the state; however, with the popularity of SPAs, something that used to be simple has become complicated.  Or has it?  Confession time: when we first attempted to sync a react component with our query string, we failed spectacularly.  At one point we weren't using query strings at all!  We started using them; after a few iterations, we had a solution but it was an over-engineered mess.  We pushed our chairs back and really *thought* about what a query string *really* represented (some kind of application state) and how that should interact with how react handles state.  And so we came up with an answer that was staring us in the face all along, and is the KISSiest solution we've seen.

Our solution builds on three simple ideas:

1. Query string state represents a special subset of component state, and needs to be accessed specially (`this.state.qstate`).
2. Because it's special, query string state also needs to have its defaults uniquely specified (in `getDefaultQuery()`) and updated specially (with `setQState()`).
3. The query string represents a special state container for the entire application, and should be able to be shared with many components.  That is, query string parameter tracking should be opt-in.

Building on these axioms, we were able to make a react component API that integrates the component's state and user's query string.  We use the fantastic React Router project internally, so the tutorial also uses it.  We don't yet offer implementations for other routers, but qs-component is designed to be extensible, so if you want support for your router, open a ticket or a pull request and let us know!
 
<!--MORE-->

For this tutorial, I wanted to demonstrate interaction with an async service, so I developed a simple express endpoint to capitalize and return its input.  You should be able to adapt the concepts in this tutorial to your code and easily apply them to any of your forms.  NOTE: the tutorial uses express exclusively; however, for simplicity's sake, the code on github also includes a browser-only setTimeout based implementation.

Here's a video of the final product:

<video controls>
  <source src="/assets/images/qs-component-cropped.webm" type="video/webm">
  <source src="/assets/images/qs-component-cropped.mp4" type="video/mp4">
  Your browser does not support the video tag.
</video>

To get started let's use the `create-react-app` npm package.  

{% highlight bash startinline %}
sudo npm install -g create-react-app
create-react-app qs-tutorial
cd qs-tutorial
{% endhighlight %}

Next, grab a few other packages. The magic though is in `qs-component`!

{% highlight bash startinline %}
npm install --save react-router qs-component express body-parser cors
{% endhighlight %}

I created a small express endpoint for my echo server.  I won't go through the whole setup (you can see that in github). But here's the endpoint to get context:

{% highlight javascript startinline %}
app.post('/api/echo', function(req, res, next) {
  var input = req.body.input;
  var uppercase  = req.body.uppercase || false;

  res.json({data: uppercase ? input.toUpperCase() : input});
});
{% endhighlight %}

Now that we have an endpoint, we can start to build our component.  First we need to create a route and mount our new component:

{% highlight javascript startinline %}
<Router history={browserHistory}>
    <Route path="/echo" component={Echo} />
</Router>
{% endhighlight %}

Now that the router knows about the component, we can build it, by extending QStateComponentReactRouterImpl (or by importing `decorateReactRouter(component : React.Component) -> QStateComponent extends component)`).

{% highlight javascript startinline %}
import React from 'react';
import { QStateComponentReactRouterImpl } from 'qs-component';

export default class Echo extends QStateComponentReactRouterImpl {
  render() {
    return <div />;
  }
}
{% endhighlight %}

The first thing we have to do is override `getDefaultQuery()` and provide the state variables from the query string we want to manage. We also specify defaults, and qs-component takes care of managing the defaults in the query string to prevent users from seeing default values while also preventing infinite redirect loops.

{% highlight javascript startinline %}
getDefaultQuery() {
    return {
      input: 'foo',
      uppercase: false
    }
}
{% endhighlight %}

Then we want a method to fetch data from the server. Let's call this fetch... and use the fetch API, seems like a good naming scheme.

As you can see we're passing the server the contents of `this.state.qstate`, which is the query string state that is managed by the form.  We're taking the results from the server call and giving it to React's `this.setState` to put the results in a regular ol' react state.  

{% highlight javascript startinline %}
fetch() {
  // we're  using react's server on port 3000, and express on port 3001 don't forget the CORS middleware
  fetch('http://localhost:3001/api/echo', {
    method: 'post',
    headers: new Headers({
      'content-type': 'application/json'
    }),
    body: JSON.stringify(this.state.qstate)
  })
  .then((res) => res.json())
  .then((results) => this.setState({results: results.data}))
}
{% endhighlight %}

Alright here's the cool stuff, the JSX that does the magic.  First let's put in the input component, make it controlled, and make it update `this.state.qstate` with `this.setQState()`

{% highlight text startinline %}
<label htmlFor="echo-input">Input</label>
<input type="text"
       value={this.state.qstate.input}
       onChange={e => this.setQState({input: e.target.value})}
       id="echo-input"
/>
{% endhighlight %}

When you type into it, see the location in your browser update? Isn't that cool!  Now let's wire up the uppercase check box.

{% highlight text startinline %}
<label>
  <input type="checkbox"
         checked={this.state.qstate.uppercase}
         onChange={(e) => this.setQState({uppercase: e.target.checked})}
  /> Uppercase</label>
{% endhighlight %}

When you click the check box, it updates the location in the browser again! Now a button to send the results to the server, and a place to see the results.

{% highlight text startinline %}
<button className="btn btn-primary" onClick={this.fetch.bind(this)}>search</button>
<div>
  {this.state.results}
</div>
{% endhighlight %}

One last important thing. Now that you have the state being managed in the query string via QState. When a user comes back to the site, with that bookmarked query string, or refreshes the page, you want to show them the results from the server again.  So simply do your `fetch` on `componentDidMount`, and you're done!

{% highlight javascript startinline %}
componentDidMount() {
    this.fetch();
}
{% endhighlight %}

So now you have a component being managed by QState!

![Alt text](/assets/images/JudgeQ.jpg)

Here is the entire component decorated with some bootstrap for some extra wow factor.  You can also find the code to this tutorial [Here](http://comingsoon)

{% highlight javascript startinline %}
import React from 'react';
import { QStateComponentReactRouterImpl } from 'qs-component';

export default class Echo extends QStateComponentReactRouterImpl {
  getDefaultQuery() {
      return {
        input: 'foo',
        uppercase: false
      }
  }

  componentDidMount() {
      this.fetch();
  }

  fetch() {
    // we're  using react's server on port 3000, and express on port 3001 don't forget the CORS middleware
    fetch('http://localhost:3001/api/echo', {
      method: 'post',
      headers: new Headers({
        'content-type': 'application/json'
      }),
      body: JSON.stringify(this.state.qstate)
    }).then((res) => res.json())
    .then((results) => this.setState({results: results.data}))
  }

  render() {
    return <div className="container">
      <h1>Echo</h1>
      <div className="row">
        <div className="col-md-6 col-sm-12">
          <div className="form-group">
            <label htmlFor="echo-input">Input</label>
            <input type="text"
                   className="form-control"
                   value={this.state.qstate.input} onChange={(e) => this.setQState({input: e.target.value})}
                   id="echo-input"
            />
          </div>
          <div className="checkbox">
            <label>
              <input type="checkbox"
                     checked={this.state.qstate.uppercase}
                     onChange={(e) => this.setQState({uppercase: e.target.checked})}
              /> Uppercase </label>
          </div>
          <div>
            <button className="btn btn-primary" onClick={this.fetch.bind(this)}>search</button>
          </div>
        </div>
      </div>
      <div className="row">
        <div className="col-md-12">
          <h2>Results</h2>
          <p className="text-primary lead">{this.state.results}</p>
        </div>
      </div>
    </div>
  }
}
{% endhighlight %}
